#!/usr/bin/env python
#
# FU_arrow_draw.py
# version 1.0 [gimphelp.org]
# last modified/tested by Paul Sherman
# Thursday, 10/22/2015 on GIMP-2.8.14
#
# edited by Paul Sherman
#
# original by Gene Cash
# GIMP Python plug-in script to make nice arrows
#
# requires that Python scripting support be compiled into GIMP with
# --enable-python option to ./configure
#
# 07-DEC-2005 CEC Written
#
#===============================================
# LICENSE
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY# without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#===============================================

from gimpfu import *
import math

default_arrow_len=20
default_arrow_aspect=0.35
default_line_thickness=2

def arrow_draw(image, drawable, arrow_len, arrow_aspect, thickness, aColor, solid):

    # attempt to retrieve details of current path
    try:
        curpath=pdb.gimp_path_get_current(image)
    except:
        pdb.gimp_message('No current path!')
        return
    try:
        ppoints=pdb.gimp_path_get_points(image, curpath)[3]
    except:
        pdb.gimp_message('Path must be a simple single-component connected path')
        return


    pdb.gimp_image_undo_group_start(image)
    foreground = pdb.gimp_context_get_foreground()
    size = pdb.gimp_context_get_brush_size()
    pdb.gimp_context_set_foreground(aColor)
    pdb.gimp_context_set_brush_size(thickness)

    x1, y1=ppoints[0:2]
    # "arrow_len" is length of arrowhead
    x2, y2, slope=pdb.gimp_path_get_point_at_dist(image, arrow_len)


    # stroke and delete current path
    pdb.gimp_path_stroke_current(image)
    pdb.gimp_path_delete(image, curpath)


    diff = thickness / 2
    pdb.gimp_image_select_ellipse(image, 0, x1, y1, 1, 1)
    pdb.gimp_selection_grow(image, thickness)
    pdb.gimp_edit_fill(drawable, 2)
    pdb.gimp_selection_none(image)

    # calculate arrowhead endpoints
    # "arrow_aspect" is width/height ratio of arrowhead
    x3=x2-(y1-y2)*arrow_aspect
    y3=y2+(x1-x2)*arrow_aspect
    x4=x2+(y1-y2)*arrow_aspect
    y4=y2-(x1-x2)*arrow_aspect

    # make arrowhead path from (x3, y3) to (x1, y1) to (x4, y4)
    pdb.gimp_path_set_points(image, curpath, 1, 24, (x3, y3, 1.0,
                                                     x3, y3, 2.0,
                                                     x1, y1, 2.0,
                                                     x1, y1, 1.0,
                                                     x1, y1, 2.0,
                                                     x4, y4, 2.0,
                                                     x4, y4, 1.0,
                                                     x4, y4, 2.0))

    # stroke and delete arrowhead path
    if solid:
        pdb.gimp_path_to_selection(image, curpath, 0, TRUE, FALSE, 0, 0)
        pdb.gimp_edit_fill(drawable, 0)
        pdb.gimp_selection_none(image)
    else:
        pdb.gimp_path_stroke_current(image)

    pdb.gimp_path_delete(image, curpath)
    pdb.gimp_context_set_foreground(foreground)
    pdb.gimp_context_set_brush_size(size)
    pdb.gimp_image_undo_group_end(image)


register(
    "arrow_draw",
    "Arrow Draw\n\nTurns a path into an arrow with an arrowhead at the starting point.",
    "",
    "Paul Sherman",
    "original - 2005 Gene Cash",
    "10/22/2015",
    "<Image>/Script-Fu/Arrow Draw",
    "*",
    [(PF_SLIDER,  'arrow_len', 'Arrowhead Length', default_arrow_len, (4, 100, 2)),
     (PF_SLIDER, 'arrow_aspect', 'Arrowhead Width %', default_arrow_aspect, (.35, .85, .05)),
    (PF_SLIDER, "thickness",  "Line Thickness", default_line_thickness, (1, 20, 1)),
    (PF_COLOR,    "aColor",    "Arrow Color", "black"),
    (PF_BOOL,  "solid",   "Solid Arrowhead?",   False)
    ],
    [],
    arrow_draw)

main()
